//--Resplendent Filefind Component v 1.01
//--(c) Copyright 1998 Resplendence Sp
//--Licensed for commerical use
//--Use of this software is subject to the License Agreement
//--as can be found in LICENSE.TXT accompanying this directory.
//-------------------------------------------------------------

unit RspFind;

interface

uses
   Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, FileCtrl;

const
   bufsize=32767;

type
  TMatchEvent=procedure(Sender:TObject; directory: String; fileinfo: TSearchRec) of object;
  TChangeDirEvent=procedure(Sender:TObject; directory: String) of object;

  TFindThread = class(TThread)
  private
    TextLength: Integer;
    foundrec: TSearchRec;
    procedure RecursiveSearchDir(dir: String);
    procedure TestIfFileInDirectory (dir: String);
    procedure TestIfFileContainsText;
    { Private declarations }
    procedure MatchFound;
    procedure WhenCompleted;
    procedure WhenDirChange;
  protected
     procedure Execute; override;
  public
    CaseSensitive: Boolean;
    Found: TMatchEvent;
    DirChange: TChangeDirEvent;
    Complete: TNotifyEvent;
    Text,filemask,directory,drives: String;
    Files: TStringList;
    Paths: TStringList;
    procedure Stop;
    constructor Create;
    destructor Destroy; override;
  end;

  TRspFind = class(TComponent)
  private
    _caseSensitive: Boolean;
    _Threadpriority: TThreadPriority;
    _Directories,_Files: TStringList;
    _text,_filemask,_drives: String;
    _onFileFound: TMatchEvent;
    _onChangeDir: TChangeDirEvent;
    _onCompleted: TNotifyEvent;

    Thread: TFindThread;
    HasRun: Boolean;
    procedure Set_Drives (str: String);
    procedure Set_Files  (list: TStringList);
    procedure Set_Directories  (list: TStringList);
    procedure Set_Text   (str: String);

  protected
    //
  public
     constructor Create(AOwner: TComponent); override;
     destructor  Destroy; override;
     procedure   Start;
     procedure   Stop;
     procedure   Pause;
     procedure   Continue;
     function    GetCDRomDrives:    String;
     function    GetRamDrives:     String;
     function    GetRemoteDrives:    String;
     function    GetFixedDrives:     String;
     function    GetRemovableDrives: String;
    //
  published
    property OnFileFound: TMatchEvent  read _onFileFound write _onFileFound;
    property OnCompleted: TNotifyEvent read _onCompleted write _onCompleted;
    property OnChangeDirectory: TChangeDirEvent read _onChangeDir write _onChangeDir;

    property CaseSensitive:  Boolean         read _caseSensitive   write _caseSensitive;
    property ContainingText: String          read _text            write Set_Text;
    property Directories:    TStringList     read _Directories     write Set_Directories;
    property Drives:         String          read _drives          write Set_Drives;
    property Files:          TStringList     read _files           write Set_Files;
    property FileMask:       String          read _filemask        write _FileMask;
    property ThreadPriority: TThreadPriority read _ThreadPriority write _ThreadPriority;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Resplendence', [TRspFind]);
end;

constructor TRspFind.Create(AOwner: TComponent);
begin
    inherited Create(AOwner);

    _ThreadPriority:=tpNormal;

    _filemask:='*.exe';
    _drives:='C';

    _Directories:=TStringList.Create;
    _Directories.Sorted:=TRUE;
    _Directories.Duplicates:=dupIgnore;

    _Files:=TStringList.Create;
end;

destructor TRspFind.Destroy;
begin
     inherited Destroy;
     if (Thread<>nil) then Stop;
     _Files.Free;
     _Directories.Free;
end;


procedure TRspFind.Start;
var
    r: Integer;
    str: String;
begin
    if (HasRun=TRUE) and (Thread<>nil) then
    begin
         Thread.Stop;
         Thread.Free;
    end;

    HasRun:=TRUE;

    Thread:=TFindThread.Create;
    Thread.Complete:=_onCompleted;
    Thread.DirChange:=_onChangeDir;
    Thread.Found:=_onFileFound;
    Thread.filemask:=_filemask;
    Thread.Paths.AddStrings(_Directories);
    Thread.Drives:=_drives;

    for r:=0 to _files.Count-1 do
    begin
        str:=_files.Strings[r];
        while (Length(str)>=1)and(str[1]='\') do
            str:=Copy(str,2,Length(str));
        if (Length(str)>=1) then Thread.Files.Add(str);
    end;

    Thread.text:=_text;
    Thread.caseSensitive:=_caseSensitive;

    Thread.Priority:=_Threadpriority;
    Thread.Found:=_onFileFound;
    Thread.Resume;
end;

procedure TRspFind.Stop;
begin
    if(Thread<>nil) then
    begin
        Thread.Stop;
    end;
end;

procedure TRspFind.Pause;
begin
    if (Thread<>nil)and(Thread.Suspended=FALSE) then
    begin
        Thread.Suspend;
    end;
end;

procedure TRspFind.Continue;
begin
    if (Thread<>nil)and(Thread.Suspended=TRUE) then
    begin
        Thread.Resume;
    end;
end;

procedure TRspFind.Set_Drives (str: String);
var
   r: Integer;
   Label Again;
begin
   str:=UpperCase(str);

   Again:
   for r:=1 to Length(str) do
   begin
       if (Integer(str[r])<Integer('A'))or(Integer(str[r])>Integer('Z'))then
       begin
           Delete (str,r,1);
           goto Again;
       end;
   end;

   _drives:=str;

end;

procedure TRspFind.Set_Text (str: String);
begin
   _text:=str;
end;

procedure TRspFind.Set_Directories (list: TStringList);
var
   v: Integer;
begin
   _Directories.Clear;
   _Directories.AddStrings(list);
   repeat
       v:=_Directories.IndexOf('');
       if (v<>-1) then _Directories.Delete(v);
    until (v=-1);
end;

procedure TRspFind.Set_Files (list: TStringList);
begin
   _Files.Clear;
   _Files.AddStrings(list);
end;



function TRspFind.GetRemoteDrives: String;
var
   r: Integer;
   buf: Array[0..4] of Char;
begin
   Result:='';
   for r:=Integer('A') to Integer('Z') do
   begin
        StrPCopy(@buf,Char(r)+':\');
        if (GetDriveType(@buf)=DRIVE_REMOTE) then
            Result:=Result+Char(r);
   end;
end;


function TRspFind.GetRemovableDrives: String;
var
   r: Integer;
   buf: Array[0..4] of Char;
begin
   Result:='';
   for r:=Integer('A') to Integer('Z') do
   begin
        StrPCopy(@buf,Char(r)+':\');
        if (GetDriveType(@buf)=DRIVE_REMOVABLE) then
            Result:=Result+Char(r);
   end;
end;

function TRspFind.GetFixedDrives: String;
var
   r: Integer;
   buf: Array[0..4] of Char;
begin
   Result:='';
   for r:=Integer('A') to Integer('Z') do
   begin
        StrPCopy(@buf,Char(r)+':\');
        if (GetDriveType(@buf)=DRIVE_FIXED) then
            Result:=Result+Char(r);
   end;
end;

function TRspFind.GetCDRomDrives: String;
var
   r: Integer;
   buf: Array[0..4] of Char;
begin
   Result:='';
   for r:=Integer('A') to Integer('Z') do
   begin
        StrPCopy(@buf,Char(r)+':\');
        if (GetDriveType(@buf)=DRIVE_CDROM) then
            Result:=Result+Char(r);
   end;
end;

function TRspFind.GetRamDrives: String;
var
   r: Integer;
   buf: Array[0..4] of Char;
begin
   Result:='';
   for r:=Integer('A') to Integer('Z') do
   begin
        StrPCopy(@buf,Char(r)+':\');
        if (GetDriveType(@buf)=DRIVE_RAMDISK) then
            Result:=Result+Char(r);
   end;
end;

//---------FindThread------------------------------------------------------

constructor TFindThread.Create;
begin
     Paths:=TStringList.Create;
     Files:=TStringList.Create;
     inherited Create(TRUE);
end;

destructor TFindThread.Destroy;
begin
     inherited Destroy;
     Files.Free;
     Paths.Free;
end;

procedure TFindThread.Execute;
var
    str: String;
    r,n,v: Integer;
begin
    TextLength:=Length(text);

    if (CaseSensitive=FALSE) then Text:=LowerCase(Text);

    for r:=1 to Length(drives) do
    begin
        if (Paths.Count>=1) then
        begin
        for n:=0 to Paths.Count-1 do
        begin
            if (Terminated=FALSE) then
            begin
                v:=Pos(':',Paths[n]);
                if (v<>0) then
                begin
                    if (v=2)and(LowerCase(drives[r])=LowerCase(Paths[n][1]))then
                        RecursiveSearchDir (Paths[n])
                end
                else
                begin
                    str:=Paths[n];
                    if (Length(str)>=1)and(str[1]<>'\') then
                        str:='\'+str;
                    RecursiveSearchDir (drives[r]+':'+str);
                end;
            end;
        end
        end
        else
        begin
            if (Terminated=FALSE) then
            begin
                RecursiveSearchDir (drives[r]+':');
            end;
        end;
    end;

    if (Terminated=FALSE) then
        Synchronize(WhenCompleted);
end;

procedure TFindThread.RecursiveSearchDir(dir: String);
var
    rec: TSearchRec;
    v: Integer;
begin
     directory:=dir;
     Synchronize(WhenDirChange);

     if (Terminated=FALSE) then
     begin
         TestIfFileInDirectory(dir);

         if (Terminated=FALSE) then
         begin
             v:=FindFirst(dir+'\*',faDirectory,rec);
             if (v=0) then
             begin
                 if Boolean(rec.Attr and $10)and(rec.Name<>'.')and(rec.Name<>'..') then
                      RecursiveSearchDir (dir+'\'+rec.Name);

                 repeat
                 v:=FindNext(rec);
                 if (v=0) and Boolean(rec.Attr and $10)and(rec.Name<>'.')and(rec.Name<>'..') then
                 begin
                     RecursiveSearchDir (dir+'\'+rec.Name);
                 end;
                 until (v<>0)or(Terminated=TRUE);
                 FindClose(rec);
             end;
         end;
     end;
end;

procedure TFindThread.TestIfFileContainsText;
var
    h: Integer;
    size,sizeleft:Integer;
    buf: Array[0..bufsize] of Byte;
    r,n: Integer;
    Label Out;
begin

    h:=FileOpen(Directory+'\'+Foundrec.Name,fmOpenRead or fmShareDenyNone);
    if (h=-1) then Exit;

    sizeleft:=Foundrec.Size;

    repeat
        if (sizeleft>bufsize) then size:=bufsize
            else size:=sizeleft;

        FileSeek(h,1,-textlength);    // Strings may appear in the gap between two blocks
        FileRead(h,buf,size);

        if (size<>sizeleft) then
            sizeleft:=sizeleft-size+textlength
        else sizeleft:=sizeleft-size;

        if (CaseSensitive=FALSE) then
        begin
            for r:=0 to size do
                if (buf[r]>=Byte('A'))and(buf[r]<=Byte('Z')) then
                    buf[r]:=buf[r]+32;
        end;


        for r:=0 to size-textlength do
        begin
            if (buf[r]=Byte(text[1])) then
            begin
                for n:=1 to textlength do
                begin
                    if (Byte(text[n])<>buf[r+n-1]) then Break;
                    if (n=textlength) then
                    begin
                        Synchronize(MatchFound);
                        goto Out;
                    end;
                end;
            end;
        end;
    until (sizeleft=0);

    Out:
    FileClose(h);

end;


procedure TFindThread.TestIfFileInDirectory (dir: String);
var
    v: Integer;
    rec: TSearchRec;
begin
    if (Terminated=FALSE) then
    begin
        v:=FindFirst(dir+'\'+filemask,$3F,rec);
        if (v=0)then
        begin
            if(rec.Name<>'.')and(rec.Name<>'..') then
            begin
                FoundRec:=rec;
                Directory:=dir;
                if (Files.Count=0)or(Files.IndexOf(rec.Name)<>-1)or(Files.IndexOf(Directory+'\'+rec.Name)<>-1)or
                        (Files.IndexOf(Copy(Directory,4,255)+'\'+rec.Name)<>-1) then
                begin

                    if (textlength>=1) then
                    begin
                        if (Foundrec.Name='ega.cpi') then
                        begin
                            Directory:=Directory+' ';
                        end;
                        if ((rec.Attr and faDirectory)=0)and(rec.Size>=textlength) then
                            TestIfFileContainsText;
                    end
                    else Synchronize(MatchFound);
                end;
            end;
            repeat
                v:=FindNext(rec);
                if (v=0)and(rec.Name<>'.')and(rec.Name<>'..') then
                begin
                    FoundRec:=rec;
                    Directory:=dir;
                    if (Files.Count=0)or(Files.IndexOf(rec.Name)<>-1)or(Files.IndexOf(Directory+'\'+rec.Name)<>-1)or
                        (Files.IndexOf(Copy(Directory,4,255)+'\'+rec.Name)<>-1) then
                    begin
                        if (textlength>=1) then
                        begin
                        if (Foundrec.Name='ega.cpi') then
                        begin
                            Directory:=Directory+'';
                        end;
                            if ((rec.Attr and faDirectory)=0)and(rec.Size>=textlength) then
                                TestIfFileContainsText;
                        end
                        else Synchronize(MatchFound);
                    end;
                end;
            until (v<>0);

            FindClose(rec);
        end;
    end;
end;

procedure TFindThread.Stop;
begin
     Terminate;
     while (Terminated=FALSE) do ;
end;

procedure TFindThread.WhenCompleted;
begin
    if (@Complete<>nil) then
        Complete(Self);
end;


procedure TFindThread.WhenDirChange;
begin
    if (Terminated=FALSE) then
        if (@DirChange<>nil) then
            DirChange(Self,directory);
end;

procedure TFindThread.MatchFound;
begin
    if (Terminated=FALSE) then
        if (@Found<>nil) then
           Found(Self,Directory,foundrec);
end;

end.





